# -*- coding: utf-8 -*-

import torch
from torch import nn, Tensor

from torchnorms.tnorms.base import BaseTNorm

from typing import Optional


class AdditiveLinearGenerator(BaseTNorm):
	def __init__(self, vec_size: int = 1) -> None:
		super().__init__()

		self.vec_size = vec_size
		self._lambda = 100
		self.function = torch.exp(torch.rand(vec_size))
		self.function =  nn.Parameter(self.function)
		# self.relu = torch.nn.ReLU()


	def __call__(self,
				 a: Tensor,
				 b: Tensor) -> Tensor:
		t_norm: Optional[Tensor] = None

		gen_a = self.function * (1 - a)
		gen_b = self.function * (1 - b)

		t_norm = gen_a + gen_b
		t_norm = torch.minimum(self.function, t_norm)
		t_norm = (self.function - t_norm) / self.function

		return t_norm


class AdditiveGenerator(BaseTNorm):
	def __init__(self, vec_size: int = 1) -> None:
		super().__init__()

		self.vec_size = vec_size
		self._lambda = 100
		self.function = torch.exp(torch.rand(vec_size,vec_size))
		self.function =  nn.Parameter(self.function)
		# self.relu = torch.nn.ReLU()


	def __call__(self,
				 a: Tensor,
				 b: Tensor) -> Tensor:
		t_norm: Optional[Tensor] = None

		gen_a = self.function @ (1 - a)
		gen_b = self.function @ (1 - b)

		t_norm = gen_a + gen_b
		t_norm = torch.minimum(torch.mean(self.function), t_norm)

		t_norm = (torch.inverse(self.function) @ (-t_norm)) + 1

		return t_norm


class AdditiveOrthogonalGenerator(BaseTNorm):
	def __init__(self, vec_size: int = 1) -> None:
		super().__init__()

		self.vec_size = vec_size
		self._lambda = 100
		self.function = torch.exp(torch.rand(vec_size,vec_size))
		self.function =  nn.Parameter(self.function)
		# self.relu = torch.nn.ReLU()


	def __call__(self,
				 a: Tensor,
				 b: Tensor) -> Tensor:
		t_norm: Optional[Tensor] = None

		regularizer = self._lambda* torch.pow(torch.norm((self.function @ self.function.T) - torch.eye(self.vec_size) ,  p='fro'), 2)

		gen_a = self.function @ (1 - a)
		gen_b = self.function @ (1 - b)

		t_norm = gen_a + gen_b

		t_norm = torch.minimum(torch.mean(self.function), t_norm)
		t_norm = (self.function.T @ (-t_norm)) + 1

		return t_norm, regularizer

# class AdditiveLinearGenerator(BaseTNorm):
# 	def __init__(self, vec_size: int = 1) -> None:
# 		super().__init__()
#
# 		self.vec_size = vec_size
# 		self._lambda = 100
# 		self.function = torch.exp(torch.randn(vec_size,vec_size))
# 		self.function =  nn.Parameter(self.function)
# 		# self.relu = torch.nn.ReLU()
#
#
# 	def __call__(self,
# 				 a: Tensor,
# 				 b: Tensor) -> Tensor:
# 		t_norm: Optional[Tensor] = None
#
# 		regularizer = torch.pow(torch.norm((self.function @ self.function.T) - torch.eye(self.vec_size) ,  p='fro'), 2)
# 		# t_norm = ((1.0 / (-self.function@(a-1) + -self.function@(b-1))) @ self.function.T) + self._lambda * regularizer
# 		gen_a = self.function @ (1 - a)
# 		gen_b = self.function @ (1 - b)
#
# 		t_norm = gen_a + gen_b
# 		t_norm = (-t_norm @ self.function.T) + 1
#
# 		return t_norm, regularizer
# #
#
# class AdditiveNonLinearGenerator(BaseTNorm):
# 	def __init__(self, vec_size: int = 1) -> None:
# 		self.vec_size = vec_size
# 		self._lambda = 100
# 		self.function = torch.normal(vec_size,vec_size)
# 		self.function =  nn.Parameter(self.function)
# 		# self.relu = torch.nn.ReLU()
# 		super().__init__()
#
#
# 	def __call__(self,
# 				 a: Tensor,
# 				 b: Tensor) -> Tensor:
# 		t_norm: Optional[Tensor] = None
#
# 		regularizer = torch.pow(torch.norm((self.function @ self.function.T) - torch.eye(self.vec_size) ,  p='fro'), 2)
#
# 		gen_a = torch.exp(self.function@a) * (1 - a)
# 		gen_b = torch.exp(self.function@b) * (1 - b)
#
# 		t_norm = gen_a+gen_b
# 		t_norm = torch.exp(self.function@(-t_norm)) * (1.0 / (1 - t_norm))
# 		return t_norm
